if (window.customElements && window.customElements.define) {
  class ISIPannable extends HTMLElement {
    constructor() {
      super();
      
      this.root = this.attachShadow({
        mode: 'open'
      });
      debugger
      
      this.initialized = false;
      this.container = document.createElement('div');
      this.container.classList = 'pan-container';
      this.scrollCss = this.scrollCss.bind(this);
      this.waitForCustomScrollFn = this.waitForCustomScrollFn.bind(this);      
      this.loadCss("mCustomScrollbar.css");
            
      const self = this;
      this.mCustomScrollbar = false;
      this.loadJs("mCustomScrollbar.concat.min.js",  () => {
        this.mCustomScrollbar = true;
      })
    }
    
    loadCss(source, callback = false){    
      let tag = document.createElement("link");
      tag.href = source;
      if(callback){
        tag.addEventListener("load", callback)
      }
      this.root.appendChild(tag);
    }
    
    loadJs(source, callback = false){
      let tag = document.createElement("script");
      tag.src = source;
      if(callback){
        tag.addEventListener("load", callback)
      }
      this.root.appendChild(tag);
    }
    
    waitForCustomScrollFn(callback) {
      if(this.mCustomScrollbar){
        callback()
      }else{
        setTimeout(() => {
          this.waitForCustomScrollFn(callback)
        }, 200);
      }
    }

    initializeAttributes() {
      this.image = this.getAttribute('image');
      this.scroll = this.hasAttribute('scroll');
      this.scrollAxis = this.getAttribute('scroll-axis');
      this.position = this.getAttribute('position');
      this.color = this.getAttribute('color');
      this.autoScroll = this.hasAttribute('auto-scroll');
      this.delay = parseInt(this.getAttribute('delay'));
      this.speed = parseInt(this.getAttribute('speed'));
      this.scrollDirection = this.getAttribute('scroll-direction');
      this.conclusionAction = this.getAttribute('conclusion-action');
      this.disable = this.hasAttribute('disable');
      this.name = this.getAttribute('prefixName');
    }
    
    scrollCss() {  
      const self = this;
      let customScroll = {
        setWidth: "100%",
        setHeight: "100%",
        callbacks: {
           onScrollStart:function(){
            let scrollStartEvent = new CustomEvent("onScrollingStart",{ detail: { "name": self.name } });
            self.dispatchEvent(scrollStartEvent)
          },
          onScroll:function(){
            let scrollEvent = new CustomEvent("onScrolling",{ detail: { "name": self.name } });
            self.dispatchEvent(scrollEvent)
          },
          onTotalScroll:function(){
            let scrollEndEvent = new CustomEvent("onScrollingEnd",{ detail: { "name": self.name } });
            self.dispatchEvent(scrollEndEvent)
          },
          onTotalScrollBack:function(){
            let scrollComplete = new CustomEvent("onScrollingComplete",{ detail: { "name": self.name } });
            self.dispatchEvent(scrollComplete)
          }
        }
      }      
      if(this.scrollAxis == "x") {
        customScroll.axis = "x"        
      } else {
        customScroll.axis = "y"
      }
      
      if(this.scroll) {
        customScroll.scrollButtons = { enable: true }
      } else {
        customScroll.scrollButtons = { enable: false };
        customScroll.autoHideScrollbar = true
      }
      
      if(this.position == "Inner") {
        customScroll.scrollbarPosition = "inside"
      } else {
        customScroll.scrollbarPosition = "outside"
      }
      
      if(this.color != undefined) {
        let scrollColor = document.createElement('style');
        scrollColor.innerHTML = `
          .mCSB_scrollTools .mCSB_draggerRail {
            background-color: ${this.color} !important
          }
        `
        this.appendChild(scrollColor)
      }
      
      if(this.speed != undefined) {
        customScroll.mouseWheel = { scrollAmount: this.speed }
      }
      
      if(this.conclusionAction == "Return (to original positon)") {
        customScroll.callbacks = {
          onTotalScrollBack:function(){
              $(this).find(".pan-container").mCustomScrollbar("scrollTo","top",{
                timeout:3000
              });
              $(this).find(".pan-container").mCustomScrollbar("disable");
          }
        }
      }
      else if(this.conclusionAction == "Loop"){
          var maxScroll = this.scrollHeight - this.clientHeight;
          var currentScroll = this.scrollTop;
          var bottomScroll = 100;
          if ( currentScroll + bottomScroll >= maxScroll ) {
            customScroll.callbacks = {
              whileScrolling:function(){
                if(this.mcs.top>=this.scrollHeight){
                  $(this).find(".pan-container").mCustomScrollbar("scrollTo","top",{
                    timeout:1000
                  });
                }else{
                  $(this).find(".pan-container").mCustomScrollbar("scrollTo","bottom",{
                    timeout:1000
                  });
                }
              }
            }
          }
      }
      
      if(this.scrollDirection == "Down") {
          customScroll.axis = "y"
      } else if(this.scrollDirection == "Up") {
           $(this).find(".pan-container").mCustomScrollbar("scrollTo","bottom",{
                timeout:1000
           });
           customScroll.axis = "y"
      } else if(this.scrollDirection == "Right") {
          customScroll.axis = "x"
      } else {
          customScroll.setLeft = "80px";
          customScroll.axis = "x"
      }
      
      if (this.disable) {
                const self = this;
                var activityEvents = [
                    'mousedown', 'mousemove', 'keydown',
                    'scroll', 'touchstart'
                ];
                activityEvents.forEach(function(eventName) {
                    self.addEventListener(eventName, function() {
                        setTimeout(() => {
                            $(self.root).find(".pan-container").mCustomScrollbar("disable"); 
                        }, 100)
                    }, true);
                });
            }     
      $((this.root).querySelector(".pan-container")).mCustomScrollbar(customScroll)        
      
      if(this.autoScroll) {
        const self = this;   
/*        let auto = setInterval(() => {
          let pos = (self.root).querySelector(".mCSB_container").offset().top;
        
          var _this = (self.root).querySelector(".img-css")
          let imageHeight = _this.height();      
        
          if(((pos - 136) *-1) <= imageHeight) {
            (self.root).querySelector(".mCSB_container").offset({top: pos - 140});
          } else {
            (self.root).querySelector(".mCSB_container").offset({top: -2650});
            clearInterval(auto);
          }
        }, 1000) */
        
      }
      
    }  
    
    
    addImage() {
      const children = this.children;   
      this.elements = this.children.length;
      this.panDiv = document.createElement('div');
      this.panDiv.className = "pannable-div";
      this.container.innerHTML = `
      <style>
        .pan-container{
          position: relative;
          width: 100%;
          height: 100%;
          overflow: scroll;
        }        
       </style>
      `
      
      for(let i=0; i<this.elements; i++) {
        const slot = document.createElement('slot');
        slot.className = "pannable-slot";
        slot.name = `element_${Math.random().toString(36).substring(4)}_${i}`
        children[i].slot = slot.name;
        this.panDiv.appendChild(slot);
        this.container.appendChild(this.panDiv);
      }       
      
      this.root.appendChild(this.container);
    }  
    
    connectedCallback() {
      const self = this;
      this.initializeAttributes();
      self.waitForCustomScrollFn(() => {
        if (!self.initialized) {
          self.addImage();
          self.initialized = true;
        }
        self.scrollCss();
      })
    }
    
    attributeChangedCallback() {
      const self = this;
      this.initializeAttributes();
      setTimeout(() => {
        if (!self.initialized) {
          self.initialized = true;
        }
      }, 10)
    }
    
    getMaxScroll() {
      return Math.max( $(this).find(".pan-container").scrollHeight, $(this).find(".pan-container").offsetHeight, 
      $(this).find(".pan-container").clientHeight, $(this).find(".pan-container").scrollHeight,  $(this).find(".pan-container").offsetHeight );
    }
    
    panScrollTo(panLeft, panTop, panAnimate, pandDuration, panEasing, panCallback) {
       $(this).find(".pan-container").mCustomScrollbar("scrollTo","left",{
          timeout:panLeft
       });
       $(this).find(".pan-container").mCustomScrollbar("scrollTo","top",{
          timeout:panTop
       });
        $(this).find(".pan-container").mCustomScrollbar("scrollTo","left",{
          advanced:{ autoUpdateTimeout: pandDuration }
       });
       $(this).find(".pan-container").mCustomScrollbar("scrollTo","left",{
        scrollEasing: panEasing 
      });
      $(this).find(".pan-container").mCustomScrollbar("scrollTo","left",{
        callbacks: panCallback
      });
      $(this).find(".pan-container").mCustomScrollbar("scrollTo","left",{
        scrollEasing: panAnimate
      });
    }
    
    getCurrPosition() {
        $(this).find(".pan-container").mcs.left;
        $(this).find(".pan-container").mcs.top;
        $(this).find(".pan-container").mcs.topPct;
        $(this).find(".pan-container").mcs.leftPct;
    }

  }
  window.customElements.define("airtory-isi-pannable", ISIPannable)
}